package main;

use 5.006002;

use strict;
use warnings;

use Date::Tolkien::Shire::Data qw{ __format __on_date };
use Test::More 0.47;	# The best we can do with Perl 5.6.2.

plan tests => 150;

my $normal = {
    year	=> 1419,
    month	=> 3,
    day		=> 25,
};
my $holiday	= {
    year	=> 1419,
#   month	=> 0,
#   day		=> 2,
    holiday	=> 2,
};
my $special	= {
    year	=> 1419,
#   month	=> 0,
#   day		=> 3,
    holiday	=> 3,
    hour	=> 1,
    minute	=> 2,
    second	=> 3,
    epoch	=> 1234,
    offset	=> 0,
    time_zone_short_name	=> 'UTC',
};

is( __format( $normal,  '%{year}' ), '1419', q<%{year} on 25 Rethe 1419> );
is( __format( $holiday, '%{year}' ), '1419', q<%{year} on 1 Lithe 1419> );
is( __format( $special, '%{year}' ), '1419', q<%{year} on Midyear's day 1419> );

is( __format( $normal,  '%A' ), 'Sunday', q<%A on 25 Rethe 1419> );
is( __format( $holiday, '%A' ), 'Highday', q<%A on 1 Lithe 1419> );
is( __format( $special, '%A' ), '', q<%A on Midyear's day 1419> );

is( __format( $normal,  '%a' ), 'Sun', q<%a on 25 Rethe 1419> );
is( __format( $holiday, '%a' ), 'Hig', q<%a on 1 Lithe 1419> );
is( __format( $special, '%a' ), '', q<%a on Midyear's day 1419> );

is( __format( $normal,  '%B' ), 'Rethe', q<%B on 25 Rethe 1419> );
is( __format( $holiday, '%B' ), '', q<%B on 1 Lithe 1419> );
is( __format( $special, '%B' ), '', q<%B on Midyear's day 1419> );

is( __format( $normal,  '%b' ), 'Ret', q<%b on 25 Rethe 1419> );
is( __format( $holiday, '%b' ), '', q<%b on 1 Lithe 1419> );
is( __format( $special, '%b' ), '', q<%b on Midyear's day 1419> );

is( __format( $normal,  '%C' ), '14', q<%C on 25 Rethe 1419> );
is( __format( $holiday, '%C' ), '14', q<%C on 1 Lithe 1419> );
is( __format( $special, '%C' ), '14', q<%C on Midyear's day 1419> );

# %c is deferred until all component formats have been tested.

# %D is deferred until all component formats have been tested.

is( __format( $normal,  '%d' ), '25', q<%d on 25 Rethe 1419> );
is( __format( $holiday, '%d' ), '02', q<%d on 1 Lithe 1419> );
is( __format( $special, '%d' ), '03', q<%d on Midyear's day 1419> );

is( __format( $normal,  '%EA' ), 'Sunnendei', q<%EA on 25 Rethe 1419> );
is( __format( $holiday, '%EA' ), 'Highdei', q<%EA on 1 Lithe 1419> );
is( __format( $special, '%EA' ), '', q<%EA on Midyear's day 1419> );

is( __format( $normal,  '%Ea' ), 'Sun', q<%Ea on 25 Rethe 1419> );
is( __format( $holiday, '%Ea' ), 'Hig', q<%Ea on 1 Lithe 1419> );
is( __format( $special, '%Ea' ), '', q<%Ea on Midyear's day 1419> );

sub _ED {
    my ( $month, $day ) = @_;
    my $rslt = __on_date( $month, $day );
    defined $rslt
	and return "\n$rslt";
    return $rslt;
}

is( __format( $normal,  '%ED' ), _ED( 3, 25 ),
    q<%ED on 25 Rethe 1419> );
is( __format( $holiday, '%ED' ), _ED( 0, 2 ) || '',
    q<%ED on 1 Lithe 1419> );
is( __format( $special, '%ED' ), _ED( 0, 3 ),
    q<%ED on Midyear's day 1419> );

is( __format( $normal,  '%Ed' ), __on_date( 3, 25 ),
    q<%Ed on 25 Rethe 1419> );
is( __format( $holiday, '%Ed' ), __on_date( 0, 2 ) || '',
    q<%Ed on 1 Lithe 1419> );
is( __format( $special, '%Ed' ), __on_date( 0, 3 ),
    q<%Ed on Midyear's day 1419> );

is( __format( $normal,  '%EE' ), '', q<%EE on 25 Rethe 1419> );
is( __format( $holiday, '%EE' ), '1 Lithe', q<%EE on 1 Lithe 1419> );
is( __format( $special, '%EE' ), q<Midyear's day>,
    q<%EE on 1 Midyear's day 1419> );

is( __format( $normal,  '%Ee' ), '', q<%Ee on 25 Rethe 1419> );
is( __format( $holiday, '%Ee' ), '1Li', q<%Ee on 1 Lithe 1419> );
is( __format( $special, '%Ee' ), 'Myd', q<%Ee on Midyear's day 1419> );

is( __format( $normal,  '%e' ), '25', q<%e on 25 Rethe 1419> );
is( __format( $holiday, '%e' ), ' 2', q<%e on 1 Lithe 1419> );
is( __format( $special, '%e' ), ' 3', q<%e on Midyear's day 1419> );

# %F is deferred until all component formats have been tested.

is( __format( $normal,  '%G' ), '1419', q<%G on 25 Rethe 1419> );
is( __format( $holiday, '%G' ), '1419', q<%G on 1 Lithe 1419> );
is( __format( $special, '%G' ), '1419', q<%G on Midyear's day 1419> );

is( __format( $normal,  '%H' ), '00', q<%H on 25 Rethe 1419> );
is( __format( $holiday, '%H' ), '00', q<%H on 1 Lithe 1419> );
is( __format( $special, '%H' ), '01', q<%H on Midyear's day 1419> );

is( __format( $normal,  '%h' ), 'Ret', q<%h on 25 Rethe 1419> );
is( __format( $holiday, '%h' ), '', q<%h on 1 Lithe 1419> );
is( __format( $special, '%h' ), '', q<%h on Midyear's day 1419> );

is( __format( $normal,  '%I' ), '12', q<%I on 25 Rethe 1419> );
is( __format( $holiday, '%I' ), '12', q<%I on 1 Lithe 1419> );
is( __format( $special, '%I' ), '01', q<%I on Midyear's day 1419> );

is( __format( $normal,  '%j' ), '086', q<%j on 25 Rethe 1419> );
is( __format( $holiday, '%j' ), '182', q<%j on 1 Lithe 1419> );
is( __format( $special, '%j' ), '183', q<%j on Midyear's day 1419> );

is( __format( $normal,  '%k' ), ' 0', q<%k on 25 Rethe 1419> );
is( __format( $holiday, '%k' ), ' 0', q<%k on 1 Lithe 1419> );
is( __format( $special, '%k' ), ' 1', q<%k on Midyear's day 1419> );

is( __format( $normal,  '%l' ), '12', q<%l on 25 Rethe 1419> );
is( __format( $holiday, '%l' ), '12', q<%l on 1 Lithe 1419> );
is( __format( $special, '%l' ), ' 1', q<%l on Midyear's day 1419> );

is( __format( $normal,  '%M' ), '00', q<%M on 25 Rethe 1419> );
is( __format( $holiday, '%M' ), '00', q<%M on 1 Lithe 1419> );
is( __format( $special, '%M' ), '02', q<%M on Midyear's day 1419> );

is( __format( $normal,  '%m' ), '03', q<%m on 25 Rethe 1419> );
is( __format( $holiday, '%m' ), '00', q<%m on 1 Lithe 1419> );
is( __format( $special, '%m' ), '00', q<%m on Midyear's day 1419> );

is( __format( $normal,  '%n' ), "\n", q<%n on 25 Rethe 1419> );
is( __format( $holiday, '%n' ), "\n", q<%n on 1 Lithe 1419> );
is( __format( $special, '%n' ), "\n", q<%n on Midyear's day 1419> );

is( __format( $normal,  '%N' ), '000000000', q<%N on 25 Rethe 1419> );
is( __format( $normal,  '%3N' ), '000', q<%3N on 25 Rethe 1419> );
is( __format( $normal,  '%0N' ), '000000000', q<%0N on 25 Rethe 1419> );

is( __format( $normal,  '%P' ), 'am', q<%P on 25 Rethe 1419> );
is( __format( $holiday, '%P' ), 'am', q<%P on 1 Lithe 1419> );
is( __format( $special, '%P' ), 'am', q<%P on Midyear's day 1419> );

is( __format( $normal,  '%p' ), 'AM', q<%p on 25 Rethe 1419> );
is( __format( $holiday, '%p' ), 'AM', q<%p on 1 Lithe 1419> );
is( __format( $special, '%p' ), 'AM', q<%p on Midyear's day 1419> );

is( __format( $normal,  '%R' ), '00:00', q<%R on 25 Rethe 1419> );
is( __format( $holiday, '%R' ), '00:00', q<%R on 1 Lithe 1419> );
is( __format( $special, '%R' ), '01:02', q<%R on Midyear's day 1419> );

# %r is deferred until all component formats have been tested.

is( __format( $normal,  '%S' ), '00', q<%S on 25 Rethe 1419> );
is( __format( $holiday, '%S' ), '00', q<%S on 1 Lithe 1419> );
is( __format( $special, '%S' ), '03', q<%S on Midyear's day 1419> );

is( __format( $normal,  '%s' ), '', q<%s on 25 Rethe 1419> );
is( __format( $holiday, '%s' ), '', q<%s on 1 Lithe 1419> );
is( __format( $special, '%s' ), '1234', q<%s on Midyear's day 1419> );

is( __format( $normal,  '%T' ), '00:00:00', q<%T on 25 Rethe 1419> );
is( __format( $holiday, '%T' ), '00:00:00', q<%T on 1 Lithe 1419> );
is( __format( $special, '%T' ), '01:02:03', q<%T on Midyear's day 1419> );

is( __format( $normal,  '%t' ), "\t", q<%t on 25 Rethe 1419> );
is( __format( $holiday, '%t' ), "\t", q<%t on 1 Lithe 1419> );
is( __format( $special, '%t' ), "\t", q<%t on Midyear's day 1419> );

is( __format( $normal,  '%U' ), '13', q<%U on 25 Rethe 1419> );
is( __format( $holiday, '%U' ), '26', q<%U on 1 Lithe 1419> );
is( __format( $special, '%U' ), '00', q<%U on Midyear's day 1419> );

is( __format( $normal,  '%u' ), '2', q<%u on 25 Rethe 1419> );
is( __format( $holiday, '%u' ), '7', q<%u on 1 Lithe 1419> );
is( __format( $special, '%u' ), '0', q<%u on Midyear's day 1419> );

is( __format( $normal,  '%V' ), '13', q<%V on 25 Rethe 1419> );
is( __format( $holiday, '%V' ), '26', q<%V on 1 Lithe 1419> );
is( __format( $special, '%V' ), '00', q<%V on Midyear's day 1419> );

is( __format( $normal,  '%W' ), '13', q<%W on 25 Rethe 1419> );
is( __format( $holiday, '%W' ), '26', q<%W on 1 Lithe 1419> );
is( __format( $special, '%W' ), '00', q<%W on Midyear's day 1419> );

is( __format( $normal,  '%w' ), '2', q<%w on 25 Rethe 1419> );
is( __format( $holiday, '%w' ), '7', q<%w on 1 Lithe 1419> );
is( __format( $special, '%w' ), '0', q<%w on Midyear's day 1419> );

# %X is deferred until all component formats have been tested.

# %x is deferred until all component formats have been tested.

is( __format( $normal,  '%Y' ), '1419', q<%Y on 25 Rethe 1419> );
is( __format( $holiday, '%Y' ), '1419', q<%Y on 1 Lithe 1419> );
is( __format( $special, '%Y' ), '1419', q<%Y on Midyear's day 1419> );

is( __format( $normal,  '%y' ), '19', q<%y on 25 Rethe 1419> );
is( __format( $holiday, '%y' ), '19', q<%y on 1 Lithe 1419> );
is( __format( $special, '%y' ), '19', q<%y on Midyear's day 1419> );

is( __format( $normal,  '%Z' ), '', q<%Z on 25 Rethe 1419> );
is( __format( $holiday, '%Z' ), '', q<%Z on 1 Lithe 1419> );
is( __format( $special, '%Z' ), 'UTC', q<%Z on Midyear's day 1419> );

is( __format( $normal,  '%z' ), '', q<%z on 25 Rethe 1419> );
is( __format( $holiday, '%z' ), '', q<%z on 1 Lithe 1419> );
is( __format( $special, '%z' ), '+0000', q<%z on Midyear's day 1419> );

is( __format( $normal,  '%%' ), '%', q<%% on 25 Rethe 1419> );
is( __format( $holiday, '%%' ), '%', q<%% on 1 Lithe 1419> );
is( __format( $special, '%%' ), '%', q<%% on Midyear's day 1419> );

# Deferred

is( __format( $normal,  '%D' ), '03/25/19', q<%D on 25 Rethe 1419> );
is( __format( $holiday, '%D' ), '1Li/19', q<%D on 1 Lithe 1419> );
is( __format( $special, '%D' ), 'Myd/19', q<%D on Midyear's day 1419> );

is( __format( $normal,  '%F' ), '1419-03-25', q<%F on 25 Rethe 1419> );
is( __format( $holiday, '%F' ), '1419-1Li', q<%F on 1 Lithe 1419> );
is( __format( $special, '%F' ), '1419-Myd', q<%F on Midyear's day 1419> );

is( __format( $normal,  '%r' ), '12:00:00 AM', q<%r on 25 Rethe 1419> );
is( __format( $holiday, '%r' ), '12:00:00 AM', q<%r on 1 Lithe 1419> );
is( __format( $special, '%r' ), '01:02:03 AM', q<%r on Midyear's day 1419> );

is( __format( $normal,  '%X' ), '12:00:00 AM', q<%X on 25 Rethe 1419> );
is( __format( $holiday, '%X' ), '12:00:00 AM', q<%X on 1 Lithe 1419> );
is( __format( $special, '%X' ), '01:02:03 AM', q<%X on Midyear's day 1419> );

is( __format( $normal,  '%x' ), '25 Ret 1419', q<%x on 25 Rethe 1419> );
is( __format( $holiday, '%x' ), '1Li 1419', q<%x on 1 Lithe 1419> );
is( __format( $special, '%x' ), 'Myd 1419', q<%x on Midyear's day 1419> );

is( __format( $normal,  '%Ex' ), 'Sunday 25 Rethe 1419',
    q<%Ex on 25 Rethe 1419> );
is( __format( $holiday, '%Ex' ), 'Highday 1 Lithe 1419',
    q<%Ex on 1 Lithe 1419> );
is( __format( $special, '%Ex' ), q<Midyear's day 1419>,
    q<%Ex on Midyear's day 1419> );

# The below should be the same as '%Z', since %EZ has no definition.
is( __format( $normal,  '%EZ' ), '', q<%EZ on 25 Rethe 1419> );
is( __format( $holiday, '%EZ' ), '', q<%EZ on 1 Lithe 1419> );
is( __format( $special, '%EZ' ), 'UTC', q<%EZ on Midyear's day 1419> );

# Deferred even harder, since it uses stuff from the deferred list.

is( __format( $normal,  '%c' ), 'Sun 25 Ret 1419 12:00:00 AM',
    q<%c on 25 Rethe 1419> );
is( __format( $holiday, '%c' ), 'Hig 1Li 1419 12:00:00 AM',
    q<%c on 1 Lithe 1419> );
is( __format( $special, '%c' ), 'Myd 1419 01:02:03 AM',
    q<%c on Midyear's day 1419> );

1;

# ex: set textwidth=72 :
